package ru.csasq.cit_is_bot.ui.screens

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.ExtendedFloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.snapshotFlow
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.buildAnnotatedString
import androidx.navigation.NavController
import cit_is_bot.composeapp.generated.resources.Res
import cit_is_bot.composeapp.generated.resources.baseline_add_24
import io.ktor.client.HttpClient
import io.ktor.client.call.body
import io.ktor.client.plugins.contentnegotiation.ContentNegotiation
import io.ktor.client.request.get
import io.ktor.http.HttpStatusCode
import io.ktor.serialization.kotlinx.json.json
import kotlinx.coroutines.launch
import kotlinx.serialization.Serializable
import org.jetbrains.compose.resources.painterResource
import ru.csasq.cit_is_bot.ui.components.SwitchButton
import ru.csasq.cit_is_bot.ui.dialogs.ScenarioDialog

@Serializable
data class Scenario(
    val id: Int? = null,
    var name: String,
    var time: String,
    var messageNumber: String,
    var isEnabled: Boolean,
)

@Composable
fun ScenariosScreen(
    navController: NavController,
    paddingValues: PaddingValues,
    floatingActionButtonState: MutableState<(@Composable () -> Unit)?>,
) {
    val verticalScrollState = rememberScrollState()
    val floatingActionButtonExpandedState = remember {
        mutableStateOf(true)
    }
    LaunchedEffect(verticalScrollState) {
        var previousValue = 0
        snapshotFlow {
            verticalScrollState.value
        }.collect {
            floatingActionButtonExpandedState.value = it <= previousValue
            previousValue = it
        }
    }
    val scenarioList = remember {
        mutableStateListOf<Scenario>()
    }
    val client = HttpClient {
        install(ContentNegotiation) {
            json()
        }
//        install(WebSockets) {
//            contentConverter = KotlinxWebsocketSerializationConverter(Json)
//        }
    }
    LaunchedEffect(false) {
        client.launch {
            val response = client.get("https://cit.csasq.ru/api/scripts")
            when (response.status) {
                HttpStatusCode.OK -> scenarioList.addAll(response.body())
            }
//            client.webSocket("wss://cit.csasq.ru/ws/scripts") {
//                scenarioList.addAll(receiveDeserialized<List<Scenario>>())
//            }
        }
    }
    Column(
        modifier = Modifier
            .padding(
                paddingValues = paddingValues,
            )
            .fillMaxSize()
            .verticalScroll(
                state = verticalScrollState,
            ),
    ) {
        val scenarioDialogState = remember {
            mutableStateOf<Scenario?>(null)
        }
        floatingActionButtonState.value = @Composable {
            ExtendedFloatingActionButton(
                text = {
                    Text(
                        text = "Добавить сценарий",
                    )
                },
                icon = {
                    Icon(
                        painter = painterResource(Res.drawable.baseline_add_24),
                        contentDescription = null,
                    )
                },
                onClick = {
                    scenarioDialogState.value = Scenario(
                        name = "",
                        time = "",
                        messageNumber = "",
                        isEnabled = true,
                    )
                },
                expanded = floatingActionButtonExpandedState.value,
            )
        }

        scenarioList.forEach {
            val scenarioSwitchState = remember {
                mutableStateOf(it.isEnabled)
            }
            val captionAnnotatedString = buildAnnotatedString {
                append(it.time)
                append(" \u2022 ")
                append(it.messageNumber)
            }
            SwitchButton(
                title = it.name,
                caption = captionAnnotatedString.text,
                switchState = scenarioSwitchState,
                onCheckedChange = { value ->
                    scenarioSwitchState.value = value
                },
                onClick = {
                    scenarioDialogState.value = it
                },
            )
        }

        scenarioDialogState.value?.let {
            ScenarioDialog(
                scenarioDialogState = scenarioDialogState,
                client = client,
            )
        }
    }
}